      SUBROUTINE TVFVFC(A1,A2,A3,A4,A5,A6,A7,A8,A9,A10)
C     TVFVFC IS A ROUTINE TO PERFORM TIME VARYING FILTER TO ONE WINDOW.  THIS
C  IS DONE IN THE AP USING THE VECTOR FUNCTION CHAINER.
C    THE CALLING PROGRAM SHOULD CALL THIS SUBROUTINE FOR EACH WINDOW OF EACH
C  TRACE. E.G. A 5 WINDOW FILTER REQUIRES 5 CALLS TO TVFVFC.
C     TVFVFC IS FOR ZERO PHASE FILTERS ONLY.
C     THE PROCEDURE IS: PAD THE FRONT OF A SCRATCH ARRAY WITH ZEROS SO THAT ZERO
C  PHASE FILTERING CAN BE DONE (HALF A FILTER LENGTH).  THE FIRST WINDOW OF A
C  TRACE DOESN'T NEED RAMPING OR MERGING SO THE ENTIRE WINDOW CAN BE DIRECTLY
C  PUT INTO THE OUTPUT ARRAY AFTER MULTIPLYING BY THE WINDOW WEIGHT.  IF THE
C  WINDOW ISN'T THE FIRST, IT MUST BE MERGED WITH THE ADJOINING WINDOW BY
C  RAMPING THE TAIL OF THE WINDOW AND ADDING IT TO THE ADJOING (AND OVERLAPPING)
C  WINDOW.
C
C  ARGUMENTS:
C  A1 - THE AP ADDRESS OF THE INPUT DATA WINDOW
C  A2 - THE NUMBER OF SAMPLES IN THE DATA WINDOW (THE LENGTH OF INPUT)
C  A3 - THE AP ADDRESS OF THE OUTPUT (THE FILTERED DATA).
C  A4 - THE AP ADDRESS OF A SCRATCH ARRAY AT LEAST NSAMPS*2 LONG
C  A5 - THE AP ADDRESS OF THE FILTER POINTS.
C  A6 - THE NUMBER OF FILTER POINTS IN THE FILTER.
C  A7 - THE AP ADDRESS OF 3 VARIABLES:
C       A7 - THE WINDOW WEIGHT
C       A7+1 - THE FRONT RAMP INCREMENT (WEIGHT/NRAMP1)
C       A7+2 - THE BACK RAMP INCREMENT (-WEIGHT/NRAMP2) - NOTE <0!!!!!!!!
C  A8 - THE NUMBER OF SAMPLES IN THE FRONT RAMP (NRAMP1)
C  A9 - THE AP ADDRESS OF THE END RAMP TO BE APPLIED TO THE INPUT DATA
C  A10 - THE NUMBER OF POINTS IN THE END RAMP.
C
C  COPYRIGHTED BY:
C  PAUL HENKART, SCRIPPS INSTITUTION OF OCEANOGRAPHY, LA JOLLA, CA., MAY 1980
C
      IMPLICIT INTEGER*2 (A-Z)
      INPUT=A1                                                          /* THE AP ADDRESS OF THE INPUT DATA
      NSAMPS=A2                                                          /* THE NUMBER OF INPUT SAMPLES
      OUTPUT=A3                                                         /* THE AP ADDRESS OF THE OUTPUT
      SCR=A4                                                            /*  THE AP ADDRESS OF A SCRATCH ARRAY
      FILT=A5                                                           /* THE AP ADDRESS OF THE FILTER
      NFPTS=A6                                                          /* THE NUMBER OF POINTS OF THE FILTER
      WEIGHT=A7                                                         /* THE AP ADDRESS OF THE WINDOW WEIGHT, FRONT RAMP IN, END RAMP INC
      NRAMP1=A8                                                         /* THE NUMBER OF SAMPLES IN THE FRONT RAMP
      INDXR2=A9                                                         /* THE AP ADDRESS OF THE START OF THE END RAMP
      NRAMP2=A10                                                         /* THE NUMBER OF SAMPLES IN THE END RAMP
      ITEMP=NFPTS-1
      NZEROS=ITEMP/2                                                    /* THE NUMBER OF ZEROS IN THE PAD
      CALL VCLR(SCR,1,NZEROS)                                            /* PAD THE FRONT OF THE DATA FOR ZERO PHASE FILTERS!
      ISTART=SCR+NZEROS                                                  /* THE BEGINNING OF THE DATA IN THE SCRATCH ARRAY
      CALL VMOV(INPUT,1,ISTART,1,NSAMPS)                                /* MOVE DATA TO SCRATCH AFTER THE PAD
      ISTART=ISTART+NSAMPS
      CALL VCLR(ISTART,1,NZEROS)
      IFILT=FILT+NFPTS
      IFILT=IFILT-1
      N=NZEROS+NSAMPS                                                    /* THE AMOUNT TO CONVOLVE
      CALL CONV(SCR,1,IFILT,-1,SCR,1,N,NFPTS)
      INDX=SCR+NSAMPS                                                    /* THE AP ADDRESS OF A SCRATCH AREA AFTER THE FILTERED DATA
      INDX=INDX+NZEROS
C****
C****  FORM A RAMP FOR THE FRONT OF THE WINDOW AND MULTIPLY AGAINST THE DATA
C****
      IF(NRAMP1.EQ.0) GO TO 100
      INC=WEIGHT+1
      CALL VRAMP(INC,INC,INDX,1,NRAMP1)
      CALL VMUL(SCR,1,INDX,1,SCR,1,NRAMP1)
      CALL VADD(SCR,1,OUTPUT,1,OUTPUT,1,NRAMP1)
      OUTPUT=OUTPUT+NRAMP1
      SCR=SCR+NRAMP1
C****
C****   MUTIPLY THE WINDOW BY IT'S WEIGHT AND PUT IT INTO OUTPUT
C****
  100 N=NSAMPS-NRAMP2
      N=N-NRAMP1
      CALL VSMUL(SCR,1,WEIGHT,OUTPUT,1,N)
C****
C****   FORM A RAMP FOR THE END OF THE WINDOW AND MULTIPLY IT AGAINST THE DATA
C****
      IF(NRAMP2.EQ.0) GO TO 200
      OUTPUT=OUTPUT+N
      SCR=SCR+N
      INC=WEIGHT+2
      CALL VRAMP(WEIGHT,INC,INDX,1,NRAMP2)
      CALL VMUL(SCR,1,INDX,1,OUTPUT,1,NRAMP2)
  200 RETURN
      END
